/*
** pjc.c - Peter's Jukebox Controller
**
** Copyright (c) 2005 Peter Eriksson <pen@lysator.liu.se>
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>

#include "pjc.h"


char *argv0 = "pjc";

int timeout = 0;
int verbose = 0;
int debug = 0;
int pktdump = 0;

int device_fd = -1;
char *device_name = NULL;




int
main(int argc,
     char *argv[])
{
    int rc, i, j, nexti;
    

    argv0 = argv[0];
    
    if ((device_name = getenv("CHANGER")) == NULL)
	device_name = "/dev/changer";
    
    for (i = 1; i < argc && argv[i][0] == '-'; ++i)
    {
	nexti = 0;
	for (j = 1; !nexti && argv[i][j]; ++j)
	    switch (argv[i][j])
	    {
	      case 'H':
		exit(cmd_help(argc-i, argv+i));
		
	      case 'h':
		printf("Usage: %s [global options] <command> [<command options>] [<command args>]\n",
		       argv0);
		puts("Global options:\n");
		puts("  -h          Display this information");
		puts("  -H [cmd]    Display all commands / usage of command");
		puts("  --          End of global options");
		puts("  -f <dev>    Changer device");
		puts("  -t <sec>    Command timeout [seconds]");
		puts("  -d [<lev>]  Increase/[Set] debug information [level]");
		puts("  -v [<lev>]  Increase/[Set] verbose level [level]");
		puts("  -x [<len>]  Display SCSI packet data/parts of [bytes]");
		exit(0);
		
	      case '-':
		break;
		
	      case 'f':
		if (argv[i][j+1])
		{
		    device_name = strdup(argv[i]+j+1);
		    nexti = 1;
		}
		else if (i+1 < argc)
		{
		    device_name = strdup(argv[++i]);
		    nexti = 1;
		}
		else
		{
		    fprintf(stderr, "%s: missing device argument for '-f'\n", argv[0]);
		    exit(1);
		}
		break;
		
	      case 't':
		if (argv[i][j+1] && isdigit((unsigned) argv[i][j+1]))
		{
		    if (sscanf(argv[i]+j+1, "%u", &timeout) != 1)
		    {
			fprintf(stderr, "%s: invalid argument for '-t': %s\n", argv[0], argv[i]+j+1);
			exit(1);
		    }
		    nexti = 1;
		}
		else
		    ++debug;
		break;

	      case 'd':
		if (argv[i][j+1] && isdigit((unsigned) argv[i][j+1]))
		{
		    if (sscanf(argv[i]+j+1, "%u", &debug) != 1)
		    {
			fprintf(stderr, "%s: invalid argument for '-d': %s\n", argv[0], argv[i]+j+1);
			exit(1);
		    }
		    nexti = 1;
		}
		else
		    ++debug;
		break;

	      case 'v':
		if (argv[i][j+1] && isdigit((unsigned) argv[i][j+1]))
		{
		    if (sscanf(argv[i]+j+1, "%u", &verbose) != 1)
		    {
			fprintf(stderr, "%s: invalid argument for '-v': %s\n", argv[0], argv[i]+j+1);
			exit(1);
		    }
		    nexti = 1;
		}
		else
		    ++verbose;
		break;

	      case 'x':
		if (argv[i][j+1] && isdigit((unsigned) argv[i][j+1]))
		{
		    if (sscanf(argv[i]+j+1, "%u", &pktdump) != 1)
		    {
			fprintf(stderr, "%s: invalid argument for '-x': %s\n", argv[0], argv[i]+j+1);
			exit(1);
		    }
		    nexti = 1;
		}
		else
		    pktdump = -1;
		break;

	      default:
		fprintf(stderr, "%s: unknown switch '%c' in argument '%s'\n",
			argv[0], argv[i][j], argv[i]);
		exit(1);
	    }
    }

    if (verbose)
	printf("[PJC %s - Copyright (c) Peter Eriksson - Built %s %s]\n\n",
	       version, __DATE__, __TIME__);
    
    device_fd = scsi_open(device_name);
    if (device_fd < 0)
    {
	fprintf(stderr, "%s: scsi_open: %s: %s\n",
		argv[0], device_name, strerror(errno));
	exit(1);
    }

    rc = cmd_execute(argc-i, argv+i);
    if (rc == CMD_OK)
    {
	if (verbose > 1)
	    printf("Command '%s' executed successfully.\n", argv[i]);

	exit(0);
    }

    if (rc < CMD_OK)
    {
	fprintf(stderr, "%s: %s: failed with code %u: %s\n",
		argv[0], argv[i], -rc, strerror(-rc));
    }
    
    else if (rc < CMD_ERROR)
    {
	const char *ss = scsi_status2string(rc);

	
	if (rc == SCSI_STATUS_CHECK_CONDITION)
	{
	    RQS rqs;
	    size_t rqslen = sizeof(rqs);

	    
	    fprintf(stderr, "%s: %s: failed: SCSI Status 0x%02x (Check Condition) : Sense Data:\n",
		    argv[0], argv[i], rc);
	    
	    rc = scsi_request_sense(device_fd, &rqs, &rqslen);
	    if (rc != 0)
	    {
		fprintf(stderr, "%s: %s: scsi_request_sense failed with code %d\n",
			argv[0], argv[i], rc);
		exit(1);
	    }

	    print_sense(stderr, rqs);
	}
	else
	    fprintf(stderr, "%s: %s: failed: SCSI Status 0x%02x (%s)\n",
		    argv[0], argv[i],
		    rc,
		    ss ? ss : "Unknown");
    }
    
    exit(1);
}
