/*
** scsi_linux.c - Linux Generic SCSI v3.1 interface code
**
** Copyright (c) 2005 Peter Eriksson <pen@lysator.liu.se>
*/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <scsi/sg.h>

#include "pjc.h"



int
scsi_open(const char *dev)
{
    int fd, ver;
    
    while ((fd = open(dev, O_RDWR|O_NONBLOCK)) < 0 && errno == EINTR)
	;


    if (ioctl(fd, SG_GET_VERSION_NUM, &ver) < 0 && errno == EINTR)
	;

    if (ver < 30100)
    {
	while (close(fd) < 0 && errno == EINTR)
	    ;
	errno = EINVAL;
	return -1;
    }
    
    return fd;
}


int
scsi_close(int fd)
{
    int rc;
    
    while ((rc = close(fd)) < 0 && errno == EINTR)
	;

    return rc;
}


/*
** Perform a generic SCSI operation.
**
** Return values:
**   0 : Operation Successful
**  <0 : ioctl() syscall failed, value is -errno
**  >0 : SCSI Operation failed, value is SCSI Status Byte
*/
int
scsi_command(int fd,              /* File descriptor for SCSI device */
	     int dir,             /* I/O direction, SCSI_NONE, SCSI_READ, SCSI_WRITE */
	     void *cdb,           /* SCSI Command Descriptor Block */
	     size_t cdblen,       /* Length of CDB */
	     void *iobuf,         /* I/O Data Buffer */
	     size_t iobuflen,     /* Read: Max size of data to transfer, Write: Data to transfer */
	     size_t *iobufres,    /* Read: Size of data returned, Write: Data written */
	     int timeout)         /* Transaction timeout in seconds */
{
    int rc;
    struct sg_io_hdr ucb;


    if (debug)
	fprintf(stderr, "scsi_command(fd=%d, dir=%d, cdblen=%u, iobuflen=%u, timeout=%d)\n",
	       fd, dir, (unsigned int) cdblen, (unsigned int) iobuflen, timeout);
    
    if (pktdump > 1)
	print_buffer(stderr, "Command Descriptor Block", cdb,
		     pktdump < cdblen ? pktdump : cdblen);
    
    memset(&ucb, 0, sizeof(ucb));

    ucb.interface_id = 'S';

    switch (dir)
    {
      case SCSI_READ:
	ucb.dxfer_direction = SG_DXFER_FROM_DEV;
	memset(iobuf, 0, iobuflen);
	break;
	
      case SCSI_WRITE:
	if (pktdump)
	    print_buffer(stderr, "Write Data Block", iobuf,
			 pktdump < iobuflen ? pktdump : iobuflen);
	
	ucb.dxfer_direction = SG_DXFER_TO_DEV;
	break;
	
      case SCSI_NONE:
	ucb.dxfer_direction = SG_DXFER_NONE;
	break;
    }
    
    
    ucb.cmd_len = cdblen;
    ucb.mx_sb_len = 0;
    ucb.iovec_count = 0;
    ucb.dxfer_len = iobuflen;
    ucb.dxferp = iobuf;
    ucb.cmdp = cdb;
    ucb.sbp = NULL;
    ucb.timeout = timeout*1000;
    ucb.flags = SG_FLAG_DIRECT_IO; /* XXX: Ignore? */
    
    while ((rc = ioctl(fd, SG_IO, &ucb)) < 0 && errno == EINTR)
	;
    
    if (debug)
	fprintf(stderr, "scsi_command: ioctl()=%d (errno=%d)\n", rc, errno);
    
    if (rc < 0)
	return -errno;
    
    if (iobufres)
    {
	*iobufres = iobuflen-ucb.resid;
	if (debug)
	    fprintf(stderr, "scsi_command: iobufres=%u\n", (unsigned int) *iobufres);
    }
    
    if (pktdump)
	print_buffer(stderr, "Read Data Block", iobuf,
		     pktdump < iobuflen-ucb.resid ? pktdump : iobuflen-ucb.resid);
    
    if (debug)
	fprintf(stderr, "scsi_command: scsi_status=0x%02x\n", ucb.masked_status<<1);
    
    return ucb.masked_status<<1;
}



