/*
** scsi_sun.c - Sun/Solaris Generic SCSI interface code
**
** Copyright (c) 2005 Peter Eriksson <pen@lysator.liu.se>
*/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/scsi/impl/uscsi.h>

#include "pjc.h"



int
scsi_open(const char *dev)
{
    int fd;
    
    while ((fd = open(dev, O_RDONLY|O_NDELAY)) < 0 && errno == EINTR)
	;

    return fd;
}


int
scsi_close(int fd)
{
    int rc;
    
    while ((rc = close(fd)) < 0 && errno == EINTR)
	;

    return rc;
}


/*
** Perform a generic SCSI operation.
**
** Return values:
**   0 : Operation Successful
**  <0 : ioctl() syscall failed, value is -errno
**  >0 : SCSI Operation failed, value is SCSI Status Byte
*/
int
scsi_command(int fd,              /* File descriptor for SCSI device */
	     int dir,             /* I/O direction, SCSI_NONE, SCSI_READ, SCSI_WRITE */
	     void *cdb,           /* SCSI Command Descriptor Block */
	     size_t cdblen,       /* Length of CDB */
	     void *iobuf,         /* I/O Data Buffer */
	     size_t iobuflen,     /* Read: Max size of data to transfer, Write: Data to transfer */
	     size_t *iobufres,    /* Read: Size of data returned, Write: Data written */
	     int timeout)         /* Transaction timeout in seconds */
{
    int rc;
    struct uscsi_cmd ucb;
    

    if (debug)
	fprintf(stderr, "scsi_command(fd=%d, dir=%d, cdblen=%u, iobuflen=%u, timeout=%d)\n",
	       fd, dir, cdblen, iobuflen, timeout);
    
    if (pktdump > 1)
	print_buffer(stderr, "Command Descriptor Block", cdb,
		     pktdump < cdblen ? pktdump : cdblen);
    
    memset(&ucb, 0, sizeof(ucb));

    /* XXX: Fixme: Make sure we really want these flags! */
    ucb.uscsi_flags = USCSI_ISOLATE /* | USCSI_DIAGNOSE */ ;

    switch (dir)
    {
      case SCSI_READ:
	ucb.uscsi_flags |= USCSI_READ;
	memset(iobuf, 0, iobuflen);
	break;

      case SCSI_WRITE:
	if (pktdump)
	    print_buffer(stderr, "Write Data Block", iobuf,
			 pktdump < iobuflen ? pktdump : iobuflen);
	
	ucb.uscsi_flags |= USCSI_WRITE;
	break;
    }
    
  ucb.uscsi_timeout = timeout;
  
  ucb.uscsi_cdb = (caddr_t) cdb;
  ucb.uscsi_cdblen = cdblen;
  ucb.uscsi_bufaddr = (caddr_t) iobuf;
  ucb.uscsi_buflen = iobuflen;

  while ((rc = ioctl(fd, USCSICMD, &ucb)) < 0 && errno == EINTR)
      ;

  if (debug)
      fprintf(stderr,
	      "scsi_command: ioctl()=%d (errno=%d, status=%u)\n",
	      rc, errno, ucb.uscsi_status);
  
  if (rc < 0)
      return -errno;

  if (iobufres)
  {
      *iobufres = iobuflen-ucb.uscsi_resid;
      if (debug)
	  fprintf(stderr, "scsi_command: iobufres=%u\n", *iobufres);
  }

  if (pktdump && (dir == SCSI_READ || iobuflen-ucb.uscsi_resid))
      print_buffer(stderr, "Read Data Block", iobuf,
		   pktdump < iobuflen-ucb.uscsi_resid ?
		   	pktdump : iobuflen-ucb.uscsi_resid);
  
  if (debug)
      fprintf(stderr, "scsi_command: scsi_status=0x%02x\n", ucb.uscsi_status);
  
  return ucb.uscsi_status;
}



